package main

import (
	"fmt"
	"time"
	"log"
	"database/sql"
	_ "github.com/lib/pq"
)

const TIMES = 3000

func trackRunTime(start time.Time, name string) {
    elapsed := time.Since(start)
    log.Printf("%s took %s", name, elapsed)
}

func selectTest(db *sql.DB) {
	defer trackRunTime(time.Now(), "select")
	for i := 0; i < TIMES; i++ {
		rows, err := db.Query("SELECT id, bar FROM foo")
		if err != nil {
			panic(fmt.Sprintf("SELECT command failed: %v\n", err))
		}
		for rows.Next() {    // for used as while loop
			//fetch and ignore rows, this will also Close() the rows.
		}
	}
}

func updateTest(db *sql.DB) {
	defer trackRunTime(time.Now(), "update")
	for i := 0; i < TIMES; i++ {
		_, err := db.Exec("UPDATE foo SET bar=bar+1 WHERE id=1")
		if err != nil {
			panic(fmt.Sprintf("UPDATE command failed: %v\n", err))
		}
	}
}

func main() {
	db, err := sql.Open("postgres", "host=localhost dbname=t1 user=t1 password=fNfwREMqO69TB9YqE+/OzF5/k+s= sslmode=disable")
	if err != nil {
		log.Fatal(err)
	}
	selectTest(db)
	updateTest(db)
}
