#!/usr/bin/env python
# coding=utf-8

"""
run db benchmark
"""

from contextlib import contextmanager
import sys
import os
import logging

import psycopg2

from wells.debug import trace_time

logging.basicConfig(format='%(levelname)-8s %(message)s', level=logging.DEBUG)
logger = logging.getLogger('')


@trace_time()
def select_test(conn, times=3000):
    cur = conn.cursor()
    for i in range(times):
        cur.execute(u"""\
SELECT id, bar FROM foo
""")
        cur.fetchone()


@trace_time()
def update_test(conn, times=3000):
    cur = conn.cursor()
    for i in range(times):
        cur.execute(u"""\
UPDATE foo SET bar=bar+1 WHERE id=1
""")
        conn.commit()


def main():
    password = os.getenv("PG_T1_PASSWORD")
    if not password:
        logger.error(u"Please define PG_T1_PASSWORD environment variable")
    conn = psycopg2.connect(host="localhost",
                            database="t1",
                            user="t1",
                            password=password)
    logger.info("running benchmark...")
    select_test(conn)
    update_test(conn)


if __name__ == '__main__':
    main()
